#include <stdio.h>
#include <string.h>

#define MAX_NAMES       10000
#define MAX_LEN         40

size_t  read_names(char *names[]);
size_t  findlow(char *names[],
		size_t first,size_t last);
void    selection_sort(char *names[],
		       int first,int last);
void    print_names(char **names,size_t count);
int     getline(char *string,int array_size);

int     main()

{
    char    *names[MAX_NAMES];
    size_t  count;
    
    count = read_names(names);
    selection_sort(names,0,count-1);
    print_names(names,count);
    return 0;
}


/*
 * Recursive selection sort
 */
 
void    selection_sort(char *names[],
		       int first,int last)

{
    int     lowpos;
    char    *temp;
    
    if ( first < last )
    {
	/* Find smallest element */
	lowpos = findlow(names,first,last);
	printf("first = %d  low = %d\n",first,lowpos);

	/* Swap smallest with first, without moving data */
	temp = names[first];
	names[first] = names[lowpos];
	names[lowpos] = temp;
	
	/* Sort remaining elements */
	print_names(names,last+1);
	puts("==");
	selection_sort(names,first+1,last);
    }
}


/*
 * Get position of alphabetically lowest name
 * Use pointers for maximum efficiency - this is the
 * most time consuming portion of the program
 */
 
size_t  findlow(char *names[],
		size_t first,size_t last)

{
    char    **low = names+first,    /* Assume 1st is low */
	    **p,
	    **end = names+last;
    
    /* Search from second onward */
    for (p = low+1; p < end; ++p)
	if ( strcmp(*p,*low) < 0 )
	    low = p;
    
    /* Return SUBSCRIPT of lowest found */
    return low-names;
}


/*
 * Read in names from stdin.  Use pointers just
 * for practice, even though they won't improve
 * overall program performance much.
 */
 
size_t  read_names(char *names[])

{
    char    **p,
	    string[MAX_LEN+1];
    
    for (p=names; (p - names < MAX_NAMES) &&
	    (getline(string,MAX_LEN) != 0); ++p)
	*p = strdup(string);
    return p - names;
}


void    print_names(char **names,size_t count)

{
    while ( count-- > 0 )
	puts(*names++);
}


/*
 * Input a string, and null terminate.  Return
 * string length.
 */

int     getline(char *string,int array_size)

{
    char    *p = string;
    
    /* Read up to newline or to array limit */
    while ( ((*p = getchar()) != '\n') &&
	    (--array_size > 0) )
	++p;
    *p = '\0';   /* null terminate */
    return p - string;
}

