#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/sem.h>

#define BLOCK_SIZE  40

int     main(int argc,char *argv[])

{
    static char    *buff;
    int     shmid, semid,
	    mode = SHM_R|SHM_W;
    
    /* Create shared memory object */
    if ( (shmid=shmget(IPC_PRIVATE,
		       BLOCK_SIZE,mode)) == -1 )
	perror("shmget() failed");
    
    /* Map object into this process' memory space */
    if ( (void *)(buff = shmat(shmid,0,0)) == (void *)-1 )
	perror("shmat() failed");

    /* Create semaphore for locking shared memory */
    if ( (semid=semget(IPC_PRIVATE,1,mode)) == -1 )
	perror("semget() failed");
    
    /* Create second process to share memory with */
    if ( fork() != 0 )
    {
	/* Options to increment semaphore */
	struct sembuf   ops[1] = {{0,1,0}};
	
	/* Write message to shared memory */
	strcpy(buff,"Hello, clone!");
	printf("Writer has: %s\n",buff);
	
	/* Detach shared memory from process */
	shmdt(buff);
	
	/* Increment semaphore to signal release of
	   shared memory to reader */
	semop(semid,ops,1);
    }
    else
    {
	/* Options to decrement semaphore */
	struct sembuf   ops[1] = {{0,-1,0}};
	
	/* Semaphore decrement operation will block
	   the process until the semaphore is
	   incremented to something > 0 by writer */
	semop(semid,ops,1);
	
	/* Print message stored in shared memory */
	printf("Reader has: %s\n",buff);

	/* Detach shared memory from process */
	shmdt(buff);

	/* Delete shared memory object from system */
	shmctl(shmid,IPC_RMID,NULL);

	/* Delete semaphore from system */
	semctl(semid,0,IPC_RMID,0);
    }
    return 0;
}

