
/* SunOS Compile: cc prog -lsocket */
/* SCO_SV Compile: cc -v prog -lsocket */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>

#define BUFF_SIZE   40

int     home_made_pipe(int fd[]);

int     main(int argc,char *argv[])

{
    int     fd[2];
    char    buff[BUFF_SIZE+1];

    if ( home_made_pipe(fd) == 0 )
    {
	if ( fork() != 0 )
	{
	    /* Parent: reader */
	    close(fd[1]);
	    
	    /* Read message from parent */
	    read(fd[0],buff,BUFF_SIZE);
	    printf("Got the message: %s",buff);
	    close(fd[0]);
	}
	else
	{
	    /* Child: writer */
	    close(fd[0]);
	    
	    /* Get a message to send */
	    fputs("Enter a message to send: ",stdout);
	    fgets(buff,BUFF_SIZE,stdin);
	    
	    /* Send message, including nul byte */
	    write(fd[1],buff,strlen(buff)+1);
	    close(fd[1]);
	}
    }
    else
	perror("socketpair() failed");

    return 0;
}


/*
 * Home-made pipe function with informative output.
 * This function creates a pair of connected sockets
 * for local interprocess communication.
 */
 
int     home_made_pipe(int fd[2])

{
#if defined(FreeBSD) || defined(Darwin)
    struct sockaddr name0 = {4, AF_UNIX, "Barny"};
    struct sockaddr name1 = {4, AF_UNIX, "Fred"};
    struct sockaddr name3 = {7, AF_UNIX, "Nobody"}; 
#else
    struct sockaddr name0 = {AF_UNIX, "Barny"};
    struct sockaddr name1 = {AF_UNIX, "Fred"};
    struct sockaddr name3 = {AF_UNIX, "Nobody"};
#endif
    int     s0, s1, s2;
    socklen_t   namelen1 = 14, namelen3 = 14;
    
    /* Create a pair of sockets */
    s0 = socket(AF_UNIX,SOCK_STREAM,0);
    if ( s0 == -1 )
	return -1;
    
    s1 = socket(AF_UNIX,SOCK_STREAM,0);
    if ( s1 == -1 )
	return -1;
	
    /* Name the sockets */
    bind(s0,&name0,namelen1);
    bind(s1,&name1,namelen1);
    getsockname(s1,&name3,&namelen3);
    printf("Socket name = %s.\n",name3.sa_data);
    
    /* Set socket s1 to listen for connect requests */
    if ( listen(s1,1) == -1 )
    {
	perror("listen() failed");
	return -1;
    }
    
    /* Send connection request from s0 to s1 */
    if ( connect(s0,&name1,namelen1) == -1 )
    {
	perror("connect() failed");
	return -1;
    }
    
    /* Accept first connection request sent to s1 */
    if ( (s2=accept(s1,&name3,&namelen3)) == -1 )
    {
	perror("accept() failed.");
	return -1;
    }
    else
    {
	printf("Accepted connection from %s.\n",
		name3.sa_data);
	close(s1);
    }
    
    /* Return socket pair through fd[] argument */
    fd[0] = s0;
    fd[1] = s2;

    /* Must remove names before next listen() */
    unlink("Barny");
    unlink("Fred");
    return 0;
}

