#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>

#define MIN(a,b)    ((a) < (b) ? (a) : (b))

int     cp(char *,char*);

/* These are from a library, but are inserted 
   in source code form for clarity */
size_t  gcd(size_t a,size_t b);
size_t lcm(size_t a,size_t b);

int     main(int argc,char *argv[])

{
    switch(argc)
    {
	case    3:
	    cp(argv[1],argv[2]);
	    break;
	default:
	    fprintf(stderr,"Usage: %s source destination\n",argv[0]);
	    break;
    }
    return 0;
}


int     cp(char *src,char *dest)

{
    int         infile,outfile;
    struct stat instats,outstats;
    long        optimal, buff_size, bytes;
    char        *buff;
    
    /* Open source and destination files */
    if ( (infile = open(src,O_RDONLY)) == -1 )
	return -1;
    
    if ( (outfile = open(dest,O_WRONLY|O_CREAT|
			 O_TRUNC,0700)) == -1 )
	return -1;

    /* Compute optimal buffer size */
    fstat(infile,&instats);
    fstat(outfile,&outstats);
    optimal = lcm(instats.st_blksize,
		  outstats.st_blksize);
    buff_size = MIN(optimal,65536);
    buff = (char *)malloc(buff_size);
    
    /* Copy file */
    while ( (bytes = read(infile,buff,buff_size)) > 0 )
	write(outfile,buff,bytes);
    close(infile);
    close(outfile);
    free(buff);
    /* Return 0 for success */
    return bytes;
}

/* Greatest common divisor */

size_t  gcd(size_t a,size_t b)

{
    if ( a == 0 ) return b;
    if ( b == 0 ) return a;
    if ( a < b )
	return gcd(b,a);
    else
	return gcd(a, b % a);
}

/* Least common multiple */
size_t lcm(size_t a,size_t b)

{
    return a * b / gcd(a,b);
}

