#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/wait.h>

#define COMMAND_LEN 255
#define MAX_ARGS    1024
#define STAT_MAX    20

enum {FALSE, TRUE};

/* Prototypes */
int     parse_cmd(char *cmd,char *argv[],char **infile,
		  char **outfile,char **errfile);
int     redirect(char *infile,char *outfile,
		 char *errfile);

/*
 * The Skel-shell, a skeletal Unix command interpreter
 * with redirection.
 */
 
int     main(int argc,char *argv[], char *envp[])

{
    char    cmd[COMMAND_LEN+1],
	    *new_argv[MAX_ARGS],
	    *path,
	    *infile,
	    *outfile,
	    *errfile;
    int     status;
    
    /* Input commands until "exit" is entered */
    do
    {
	fputs("Skel-shell: ",stdout);
	fgets(cmd,COMMAND_LEN,stdin);
	
	/* Set up argv array for execvp */
	parse_cmd(cmd,new_argv,
		  &infile,&outfile,&errfile);
	path = new_argv[0];
	
	/* If it's an external command, fork */
	if ( strcmp(path,"exit") != 0 )
	{
	    /* Create child process */
	    if ( fork() == 0 )
	    {
		/* Perform redirection */
		redirect(infile,outfile,errfile);
		
		/* If child, run command */
		execvp(path,new_argv);
		
		/* Error: kill child */
		perror(cmd);
		exit(1);
	    }
	    else
	    {
		/* If parent, wait for child */
		wait(&status);
	    }
	}
    }   while ( strcmp(cmd,"exit") != 0 );
    return 0;
}


/*
 * Break a command into a list of arguments,
 * and separate * out redirection filenames.
 * The redirection operators * '<', '>', and '>&'
 * all must be surrounded by whitespace.
 */
 
int     parse_cmd(char *cmd,char *argv[],char **infile,
		  char **outfile,char **errfile)

{
    int     c;
    
    /* Initialize redirection filenames */
    *infile = *outfile = *errfile = NULL;
    
    /* Tokenize command */
    argv[0] = strtok(cmd," \t\n");
    for (c=1; (argv[c] = strtok(NULL," \t\n"))
	       != NULL; ++c)
    {
	if ( strcmp(argv[c],"<") == 0 )
	{
	    *infile = strtok(NULL," \t\n");
	    --c;
	}
	else if ( strcmp(argv[c],">") == 0 )
	{
	    *outfile = strtok(NULL," \t\n");
	    --c;
	}
	else if ( strcmp(argv[c],">&") == 0 )
	{
	    *errfile = strtok(NULL," \t\n");
	    --c;
	}
    }
    return 0;
}


/*
 * Perform redirection on default stdin,
 * stdout, and stderr * file * descriptors.
 */
 
int     redirect(char *infile,char *outfile,
		 char *errfile)

{
    if ( infile != NULL )
    {
	close(0);
	open(infile,O_RDONLY,0);
    }
    if ( outfile != NULL )
    {
	close(1);
	open(outfile,O_WRONLY|O_TRUNC|O_CREAT,0644);
    }
    if ( errfile != NULL )
    {
	close(2);
	open(errfile,O_WRONLY|O_TRUNC|O_CREAT,0644);
    }
    return 0;
}

