#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAX_NAMES       100
#define MAX_NAME_LEN    40

/* Prototypes */
int     strptrcmp(char **s1,char **s2);
size_t  read_list(char *name_list[],size_t max_names);
void    print_names(char *name_list[],size_t names);
void    free_names(char *name_list[],size_t names);
int     fgetline(FILE *fp,char buff[],int maxlen);

int     main(int argc,char *argv[])

{
    char    *name_list[MAX_NAMES],
	    key[MAX_NAME_LEN+1],
	    *skey[1],
	    **match;
    size_t  names;
    
    /* Read and sort name list */
    names = read_list(name_list,MAX_NAMES);
    qsort(name_list,names,sizeof(char *),(int (*)())strptrcmp);
    print_names(name_list,names);
    
    /* Get search criteria */
    fputs("Enter a name to search for: ",stdout);
    fgetline(stdin,key,MAX_NAME_LEN);
    skey[0] = key;
    
    /* Search list */
    match = bsearch(skey,name_list,names,
		    sizeof(char *),(int (*)())strptrcmp);
    if ( match != NULL )
	printf("Found: %s",*match);
    else
	puts("No match.");
    free_names(name_list,names);
    return 0;
}


/*
 * Compare strings at two locations in an array of
 * pointers to string.
 */
 
int     strptrcmp(char **s1,char **s2)

{
    return strcmp(*s1,*s2);
}


/*
 * Read list of names into a pointer array
 */
 
size_t  read_list(char *name_list[],size_t max_names)

{
    size_t  c = 0;
    char    temp[MAX_NAME_LEN+1] = "x";
    
    fputs("Enter names followed by a blank line.\n",
	    stdout);
    while ( (c<max_names) &&
	    (fgetline(stdin,temp,MAX_NAME_LEN) != 0) )
    {
	name_list[c++] = strdup(temp);
    }
    return c;
}


/*
 * Print list of names in a pointer array
 */
 
void    print_names(char *name_list[],size_t names)

{
    size_t  c;
    
    for (c=0; c<names; ++c)
	puts(name_list[c]);
}


/*
 * Free memory allocated by read_list()
 */
 
void    free_names(char *name_list[],size_t names)

{
    while ( --names > (size_t)-1 )
	free(name_list[names]);
}


int     fgetline(FILE *fp,char buff[],int maxlen)

{
    char    *p = buff, *end = buff+maxlen;
    int     ch;

    if ( feof(fp) )
	return EOF;
    
    /* Read to end of line, end of file, or maxlen characters */
    do
    {
	*p++ = ch = getc(fp);
    }   while ( (ch != EOF) && (ch != '\n') && (p < end) );
    *--p = '\0';    /* Replaced \n with \0 */
    return p-buff;  /* Return string length */
}


