#include <stdio.h>
#include <sysexits.h>
#include <stdlib.h>

#define MAX_NAME_LEN    40

/* Safe memory allocation */
#define MALLOC(nelem,type)  malloc(nelem*sizeof(type))

/* Player structure */
typedef struct player player_t;
struct player
{
    int         world_rank;
    char        first_name[MAX_NAME_LEN+1];
    char        last_name[MAX_NAME_LEN+1];
    player_t    *next;
};

/* Prototypes */
player_t  *read_list(void);
void    print_list(player_t *);
void    free_list(player_t **);

int     main()

{
    player_t  *head = NULL;
    
    head = read_list();
    print_list(head);
    free_list(&head);
    return EX_OK;
}


/*
 * Read a list into a linked list,
 * and return a pointer to head.
 */
 
player_t  *read_list()

{
    player_t  *temp, *head = NULL, new_player;
    
    while ( scanf("%s %s %d",new_player.first_name,
		    new_player.last_name,
		    &new_player.world_rank) == 3 )
    {
	/* Allocate memory for new node */
	temp = (player_t *)MALLOC(1,player_t);
	
	*temp = new_player; /* Copy to allocated node */
	temp->next = head;  /* Point new head to old */
	head = temp;        /* New node is new head */
    }
    return head;
}


/*
 * Print the contents of a linked list.
 */
 
void    print_list(player_t *ptr)

{
    /* Start at head and stop at NULL pointer */
    while ( ptr != NULL )
    {
	printf("%s %s %d\n",ptr->first_name,
			    ptr->last_name,
			    ptr->world_rank);
	ptr = ptr->next;
    }
}

/*
 * Free all nodes in the list
 */
 
void    free_list(player_t **head)

{
    player_t    *dead_head = *head,
		*next_node = *head;
    
    /* Start at head and stop at NULL pointer */
    while ( next_node != NULL )
    {
	next_node = dead_head->next;
	free(dead_head);
	dead_head = next_node;
    }
    
    /* Mark list as empty */
    *head = NULL;
}

