#include <stdio.h>
#include <sysexits.h>
#include <stdlib.h>
#include <unistd.h>

#define MAX_NAME_LEN    40

/* Use a generous list size - they're only pointers */
#define MAX_LIST_SIZE   1000

/* Safe memory allocation */
#define MALLOC(nelem,type)  malloc(nelem*sizeof(type))

/* Player structure */
typedef struct
{
    int         world_rank;
    char        first_name[MAX_NAME_LEN+1];
    char        last_name[MAX_NAME_LEN+1];
}   player_t;

/* Prototypes */
void    read_list(player_t *list[]);
void    print_list(player_t *list[]);
void    free_list(player_t *list[]);

int     main()

{
    player_t  *list[MAX_LIST_SIZE];
    
    read_list(list);
    print_list(list);
    free_list(list);
    return EX_OK;
}


/**************************************************
 * Read a list of inventory items into a pointer
 * array.  Mark the end of * the list with NULL.
 **************************************************/
 
void    read_list(player_t *list[])

{
    player_t    new_player;
    int         list_size, c = 0;
    
    /* Read list items until EOF (ctrl+D) */
    for (list_size=0; c != EOF; ++list_size)
    {
	/* If input from keyboard, prompt user */
	if ( isatty(fileno(stdin)) )
	{
	    fputs("Enter player number and name",stdout);
	    puts(" or <ctrl>+D when finished.");
	}
	
	/* Input a record */
	c = scanf("%s %s %d",
		    new_player.first_name,
		    new_player.last_name,
		    &new_player.world_rank);
 
	/* If valid entry, add to list */
	switch(c)
	{
	    case    3:
		list[list_size] = 
		    (player_t *)MALLOC(1,player_t);
		*list[list_size] = new_player;
		break;
	    case    EOF:
		break;
	    default:
		fprintf(stderr,"Invalid entry.");
	}
    }
    
    /* Mark end of list */
    list[list_size-1] = NULL;
}


/*****************************************
 * Print list contained in pointer array.
 *****************************************/
 
void    print_list(player_t *list[])

{
    player_t  **p;
    
    for (p = list; *p != NULL; ++p)
    {
	printf("%s %s %d\n",
		(*p)->first_name,
		(*p)->last_name,
		(*p)->world_rank);
    }
}


/************
 * Free list
 ************/
 
void    free_list(player_t *list[])

{
    player_t  **p;
    
    for (p=list; *p != NULL; ++p)
	free(*p);
}

