
/* FreeBSD Compile: cc -Wall -I/usr/X11R6/include prog -L/usr/X11R6/lib -lX11 */
/* Darwin Compile: cc -Wall -I/usr/X11R6/include prog -L/usr/X11R6/lib -lX11 */
/* Linux Compile: cc -Wall -I/usr/X11R6/include prog -L/usr/X11R6/lib -lX11 */
/* SunOS Compile: cc -I/usr/openwin/include prog -L/usr/openwin/lib -lX11 */
/* IRIX Compile: cc -I/usr/include/X11 prog -L/usr/lib/X11 -lX11 */
/* IRIX64 Compile: cc -I/usr/include/X11 prog -L/usr/lib/X11 -lX11 */
/* SCO_SV Compile: cc -v -I/usr/include/X11 prog -lX11 -lsocket */

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

#define WIDTH               620
#define HEIGHT              460
#define SPACING             10
#define DRAWINGS            10
#define LINES_PER_DRAWING   300
#define MSG     "Press any mouse button to continue"

void    draw_lines(Display *disp,Window win,GC gc);

int     main()

{
    /* Display */
    Display                 *disp;
    int                     screen,depth,borderwidth;
    
    /* Window stuff */
    int                     win_x,win_y;
    XSetWindowAttributes    winat;
    unsigned long           atmask;
    Window                  win,root;
    
    /* Graphics context */
    XGCValues               gcvalues;
    GC                      gc;
    unsigned long           gcmask;
    
    /* Get server connection */
    if ( (disp = XOpenDisplay(NULL)) == NULL )
    {
	fprintf(stderr,"Cannot open display.\n");
	exit(1);
    }
    
    /* Get screen ID */
    screen = DefaultScreen(disp);
    
    /* Get color capabilities */
    depth = DefaultDepth(disp,screen);
    
    /* Print information retrieved from server */
    printf("Server: %s version %d X%d R%d\n",
	ServerVendor(disp),
	VendorRelease(disp),
	ProtocolVersion(disp),
	ProtocolRevision(disp));
    printf("Color plane depth = %d\n",depth);
    printf("Display: %s dimensions: %d x %d\n",
	XDisplayName(NULL),
	DisplayWidth(disp,screen),
	DisplayHeight(disp,screen));

    /* Set up information for a new window */
    winat.border_pixel = BlackPixel(disp,screen);
    winat.background_pixel = WhitePixel(disp,screen);
    winat.override_redirect = True;
    atmask = CWBackPixel|CWBorderPixel|
	     CWOverrideRedirect;
    
    /* Get root window pointer */
    root = RootWindow(disp,screen);
    
    /* Create new window under root */
    win_x=10;
    win_y=10;
    borderwidth=2;
    win = XCreateWindow(disp,root,win_x,win_y,
			WIDTH,HEIGHT,
			borderwidth,depth,
			InputOutput,CopyFromParent,
			atmask,&winat);
    
    /* Display new window */
    XMapWindow(disp,win);
    
    /* Create graphics context for color, etc. */
    gcmask = 0L;
    gc = XCreateGC(disp,win,gcmask,&gcvalues);
    
    /* Draw funky line patterns */
    draw_lines(disp,win,gc);
    
    /* Remove window and shut down */
    XDestroyWindow(disp,win);
    XCloseDisplay(disp);
    return(0);
}

void    draw_lines(Display *disp,Window win,GC gc)

{
    /* Line drawing vars */
    int                     x1,y1,x2,y2,
			    dx1,dx2,dy1,dy2,
			    line,drawing;
    time_t                  cur_time;
    XEvent                  event;
    
    /* Ask server to send button press events */
    XSelectInput(disp,win,ButtonPressMask);

    /* Draw funky line patterns */
    time(&cur_time);
    srandom((long)cur_time);
    for (drawing=0; drawing<DRAWINGS; ++drawing)
    {
	/* Generate random spacing and endpoints */
	x1 = random() % WIDTH;
	x2 = random() % WIDTH;
	y1 = random() % HEIGHT;
	y2 = random() % HEIGHT;
	dx1 = random() % SPACING;
	dx2 = random() % SPACING;
	dy1 = random() % SPACING;
	dy2 = random() % SPACING;

	XClearWindow(disp,win);
	XFlush(disp);
	
	/* Draw lines */
	for (line=0; line<LINES_PER_DRAWING; ++line)
	{
	    XDrawLine(disp,win,gc,x1,y1,x2,y2);
	    
	    /* "Bounce" endpoints off window edges */
	    if (((x1+dx1) > WIDTH) || ((x1+dx1) < 0))
		dx1 = -dx1;
	    if (((x2+dx2) > WIDTH) || ((x2+dx2) < 0))
		dx2 = -dx2;
	    if (((y1+dy1) > HEIGHT) || ((y1+dy1) < 0))
		dy1 = -dy1;
	    if (((y2+dy2) > HEIGHT) || ((y2+dy2) < 0))
		dy2 = -dy2;
	    
	    /* Move endpoints for next line */
	    x1 += dx1;
	    x2 += dx2;
	    y1 += dy1;
	    y2 += dy2;
	}
	XDrawString(disp,win,gc,10,10,MSG,strlen(MSG));
	XFlush(disp);
	
	/* Wait for user to press a mouse button */
	XNextEvent(disp,&event);
    }
}

