
/* FreeBSD Compile: cc -Wall -I/usr/X11R6/include prog -L/usr/X11R6/lib -lXt -lX11 */
/* Darwin Compile: cc -Wall -I/usr/X11R6/include prog -L/usr/X11R6/lib -lXt -lX11 */
/* Linux Compile: cc -Wall -I/usr/X11R6/include prog -L/usr/X11R6/lib -lXt -lX11 */
/* SunOS Compile: cc -I/usr/openwin/include prog -L/usr/openwin/lib -lXt -lX11 */
/* IRIX Compile: cc -I/usr/include/X11 prog -L/usr/lib/X11 -lXt -lX11 */
/* IRIX64 Compile: cc -I/usr/include/X11 prog -L/usr/lib/X11 -lXt -lX11 */
/* SCO_SV Compile: cc -v -I/usr/include/X11 prog -lXt -lX11 -lsocket */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
/* Place sysexits after unistd to work around a bug in IRIX.  EX_OK
   is defined differently in these two headers, and we want the
   sysexists version to override here */
#include <sysexits.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Text.h>

#define WIDTH               620
#define HEIGHT              460
#define SPACING             10
#define DRAWINGS            10
#define LINES_PER_DRAWING   300
#define MSG     "Press any mouse button to continue"

/* Prototypes */
void    draw_line(Widget canvas,caddr_t client_data,
		  XEvent *event);
    
int     main(int argc,char *argv[])

{
    /* Widget stuff */
    Widget      toplevel, canvas;
    static Arg  wargs[] =
		{
		    { XtNwidth, WIDTH },
		    { XtNheight, HEIGHT }
		};
    
    /* Graphics context */
    XGCValues       gcvalues;
    GC              gc;
    unsigned long   gcmask;

    XEvent      event;      /* For XtNextEvent() */
    int         drawing;    /* Drawing counter */
    time_t      cur_time;   /* Random number seed */

    /* Initialize intrinsics; connect to X server */
    toplevel = XtInitialize("xdemo","Xdemo",NULL,0,
			    &argc,argv);

    /* Create generic widget for drawing on */
    canvas = XtCreateManagedWidget("lines",widgetClass,
	    toplevel,wargs,XtNumber(wargs));

    /* Register event handler for button press */
    XtAddEventHandler(canvas,ButtonPressMask,FALSE,
		    (void (*)())draw_line,&gc);
    
    /* Create window for widget */
    XtRealizeWidget(toplevel);
    
    /* Create graphics context for color, etc. */
    gcmask = 0L;
    gc = XCreateGC(XtDisplay(canvas),XtWindow(canvas),
		    gcmask,&gcvalues);

    /* Seed random number generator */
    time(&cur_time);
    srandom((long)cur_time);

    /* Do 10 line drawings */
    drawing = 0;
    while (drawing < DRAWINGS)
    {
	/* Wait for next event */
	XtNextEvent(&event);
	
	/* If button press, do a drawing */
	if ( event.type == ButtonPress )
	{
	    XtDispatchEvent(&event);
	    ++drawing;
	}
    }
    return EX_OK;
}


/*
 * Do a funky line drawing
 */
 
void    draw_line(Widget canvas,caddr_t client_data,
		  XEvent *event)

{
    /* Line drawing vars */
    int     x1,y1,x2,y2,
	    dx1,dx2,dy1,dy2,
	    line;
    
    /* Get display and window for Xlib calls */
    Display *disp = XtDisplay(canvas);
    Window  win = XtWindow(canvas);
    
    /* Get graphics context from extra client data */
    GC      *gc = (GC *)client_data;
    
    /* Generate random spacing and endpoints */
    x1 = random() % WIDTH;
    x2 = random() % WIDTH;
    y1 = random() % HEIGHT;
    y2 = random() % HEIGHT;
    dx1 = random() % SPACING;
    dx2 = random() % SPACING;
    dy1 = random() % SPACING;
    dy2 = random() % SPACING;

    XClearWindow(disp,win);
    XFlush(disp);
    
    /* Draw lines */
    for (line=0; line<LINES_PER_DRAWING; ++line)
    {
	XDrawLine(disp,win,*gc,x1,y1,x2,y2);
	
	/* "Bounce" endpoints off window edges */
	if (((x1+dx1) > WIDTH) || ((x1+dx1) < 0))
	    dx1 = -dx1;
	if (((x2+dx2) > WIDTH) || ((x2+dx2) < 0))
	    dx2 = -dx2;
	if (((y1+dy1) > HEIGHT) || ((y1+dy1) < 0))
	    dy1 = -dy1;
	if (((y2+dy2) > HEIGHT) || ((y2+dy2) < 0))
	    dy2 = -dy2;
	
	/* Move endpoints for next line */
	x1 += dx1;
	x2 += dx2;
	y1 += dy1;
	y2 += dy2;
    }
    XDrawString(disp,win,*gc,10,10,MSG,strlen(MSG));
    XFlush(disp);
}

